function create_template, datastart = datastart, $          ; default 0L
                          delimiter = delimiter, $          ; default ' '
                          missingvalue = missingvalue, $    ; default NaN
                          commentsymbol = commentsymbol, $  ; default '#'
                          fieldtypes = fieldtypes, $        ; default undefined
                          fieldnames = fieldnames, $        ; default FIELD#####
                          fieldlocations = fieldlocations, $
                          filename = filename

; check input
  if (keyword_set(filename)) then begin
    template = ascii_template(filename)
    names = tag_names(template)
    for tag = 0, n_tags(template) - 1 do begin
      print, names[tag], template.(tag)
    endfor
    return, template
  endif else begin
    if (~ keyword_set(fieldlocations)) then begin
      stop, 'fieldlocations must be specified. Exiting...'
    endif
    fieldcount = n_elements(fieldlocations)

; set values
    version = 1.
    if (~ keyword_set(datastart)) then datastart = 0L
    if (~ keyword_set(delimiter)) then delimiter = ' '
    if (~ keyword_set(missingvalue)) then missingvalue = !values.f_nan
    if (~ keyword_set(commentsymbol)) then commentsymbol = '#'
    if (~ keyword_set(fieldtypes)) then begin
      fieldtypes = replicate(0, fieldcount) ; default is undefined
    endif else begin
      if (n_elements(fieldtypes) ne fieldcount) then $
        stop, 'n_elements(fieldtypes) ne fieldcount. Exiting...'
      for field = 0L, fieldcount - 1L do begin
        case fieldtypes[field] of
          'undefined': type_id = 0L ; default
          'byte': type_id = 1L
          'integer': type_id = 2L
          'long': type_id = 3L
          'float': type_id = 4L
          'double': type_id = 5L
          'complex': type_id = 6L
          'string': type_id = 7L
          'structure': type_id = 8L
          'dcomplex': type_id = 9L
          'pointer': type_id = 10L
          'object': type_id = 11L
          'uinteger': type_id = 12L
          'ulong': type_id = 13L
          'long64': type_id = 14L
          'ulong64': type_id = 15L
          else: begin
            stop, 'Could not assign ' + fieldtypes[field] + '. Exiting...'
          end
        endcase
        if (n_elements(fieldtypes_id) eq 0) then begin
          fieldtypes_id = [type_id]
        endif else begin
          fieldtypes_id = [fieldtypes_id, type_id]
        endelse
      endfor ; field
    endelse
    if (~ keyword_set(fieldnames)) then begin
      fieldnames = 'FIELD' + string(lindgen(fieldcount), format = '(i5.5)')
    endif else begin
      if (n_elements(fieldnames) ne fieldcount) then $
        stop, 'n_elements(fieldnames) ne fieldcount. Exiting...'
    endelse
    if (~ keyword_set(fieldgroups)) then fieldgroups = lindgen(fieldcount)

; create template
    return, {version: version, $
             datastart: datastart, $
             delimiter: delimiter, $
             missingvalue: missingvalue, $
             commentsymbol: commentsymbol, $
             fieldcount: fieldcount, $
             fieldtypes: fieldtypes_id, $
             fieldnames: fieldnames, $
             fieldlocations: fieldlocations, $
             fieldgroups: fieldgroups}
  endelse

end
