function read_igor, filename
  if (~ file_test(filename, /regular)) then begin
    stop, 'The file ' + filename + ' does not exist. Exiting...'
  endif

;-------------------------------------------------------------------------------
; create metadata
;-------------------------------------------------------------------------------
  tt = ['waves', 'stamp', 'time', 't_series', 'tseries', 't_60', 'msa', $
        'org', 'so4', 'no3', 'nh4', 'amm', 'cl', 'chl', 'sul', 'nitr', 'tot']
  line = ''
  openr, igor, filename, /get_lun

; read IGOR line
  readf, igor, line
  line_curr = 0L
  if ~ strcmp(line, 'IGOR') then begin
    print, line
    stop, 'The file ' + filename + ' is not in IGOR format. Exiting...'
  endif

  while ~ eof(igor) do begin

; read title line(s)
    readf, igor, line
    line_curr += 1L
    titles_curr = strsplit(line, '	', /extract)
    if (strpos(titles_curr[0L], 'WAVES') eq -1L) then continue
    if (n_elements(titles) eq 0L) then begin
      titles = titles_curr
      ncolumns = [n_elements(titles_curr)]
    endif else begin
      titles = [titles, titles_curr]
      ncolumns = [ncolumns, n_elements(titles_curr)]
    endelse

; read BEGIN line(s)
    readf, igor, line
    line_curr += 1L
    if ~ strcmp(line, 'BEGIN') then begin
      print, line
      stop, 'The file ' + filename + ' should now have the BEGIN. Exiting...'
    endif
    if (n_elements(begin_lines) eq 0L) then begin
      begin_lines = [line_curr + 1L]
    endif else begin
      begin_lines = [begin_lines, line_curr + 1L]
    endelse

; read up to the END line(s)
    while (~ strcmp(line, 'END')) do begin
      readf, igor, line
      line_curr += 1L
    endwhile
    if (n_elements(end_lines) eq 0L) then begin
      end_lines = [line_curr - 1L]
    endif else begin
      end_lines = [end_lines, line_curr - 1L]
    endelse

  endwhile
  free_lun, temporary(igor)

; set column name(s)
  columns = replicate('', n_elements(titles))
  for t = 0L, n_elements(tt) - 1L do begin
    ti = stregex(titles, tt[t], /boolean, /fold_case)
    mask = where(ti eq 1L, count)
    if (count gt 0L) then begin
      case tt[t] of
        'waves': columns[mask] = 'waves'
        'stamp': columns[mask] = 'time'
        't_series': columns[mask] = 'time'
        'tseries': columns[mask] = 'time'
        't_60': columns[mask] = 'time'
        'org': columns[mask] = 'oa'
        'chl': columns[mask] = 'cl'
        'amm': columns[mask] = 'nh4'
        'sul': columns[mask] = 'so4'
        'nitr': columns[mask] = 'no3'
        'tot': columns[mask] = 'total'
        else: columns[mask] = tt[t]
      endcase
    endif ; count eq 1L
  endfor ; t

;-------------------------------------------------------------------------------
; read data
;-------------------------------------------------------------------------------
; create template
  for dataset = 0L, n_elements(begin_lines) - 1L do begin
    if (dataset eq 0L) then begin
      fieldnames = columns[0L:ncolumns[dataset] - 1L]
    endif else begin
      fieldnames = columns[ncolumns[dataset - 1L]:$
                           total(ncolumns[0L:dataset]) - 1L]
    endelse
    fieldtypes = replicate('float', ncolumns[dataset])
    mask = where(strcmp(fieldnames, 'waves'), count)
    if (count gt 0L) then fieldtypes[mask] = 'undefined'
    mask = where(strcmp(fieldnames, 'time'), count)
    if (count gt 0L) then fieldtypes[mask] = 'string'
    template = create_template(datastart = begin_lines[dataset], $
                               delimiter = '	', $
                               fieldtypes = fieldtypes, $
                               fieldnames = fieldnames, $
                               fieldlocations = lindgen(ncolumns[dataset]))
    data_curr = read_ascii(filename, template = temporary(template), $
                           num_records = end_lines[dataset] - $
                                         begin_lines[dataset] + 1L)
    if (n_tags(data) eq 0L) then begin
      data = temporary(data_curr)
    endif else begin
      data = create_struct(data, temporary(data_curr))
    endelse
  endfor ; dataset

  return, data
end
